/*! @name @ptv.js/videojs-ptv @version 1.5.0 @license MIT */
(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('video.js'), require('global/window')) :
  typeof define === 'function' && define.amd ? define(['exports', 'video.js', 'global/window'], factory) :
  (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global.videojsPtv = {}, global.videojs, global.window));
}(this, (function (exports, videojs, window) { 'use strict';

  function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

  var videojs__default = /*#__PURE__*/_interopDefaultLegacy(videojs);
  var window__default = /*#__PURE__*/_interopDefaultLegacy(window);

  function _assertThisInitialized(self) {
    if (self === void 0) {
      throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
    }

    return self;
  }

  var assertThisInitialized = _assertThisInitialized;

  function _inheritsLoose(subClass, superClass) {
    subClass.prototype = Object.create(superClass.prototype);
    subClass.prototype.constructor = subClass;
    subClass.__proto__ = superClass;
  }

  var inheritsLoose = _inheritsLoose;

  function _defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  function _createClass(Constructor, protoProps, staticProps) {
    if (protoProps) _defineProperties(Constructor.prototype, protoProps);
    if (staticProps) _defineProperties(Constructor, staticProps);
    return Constructor;
  }

  var createClass = _createClass;

  /**
   * API hosts.
   * @typedef {Object} ApiHosts
   * @enum
   */
  var ApiHosts = Object.freeze({
    COMDEV: 'comdev',
    QA: 'qa',
    STAGING: 'staging',
    PRODUCTION: 'prod'
  });
  /**
   * Embed hosts.
   * @typedef {Object} EmbedHosts
   * @enum
   */

  var EmbedHosts = Object.freeze({
    COMDEV: 'comdev.embed.promethean.tv',
    QA: 'qa.embed.promethean.tv',
    STAGING: 'staging.embed.promethean.tv',
    PRODUCTION: 'embed.promethean.tv'
  });
  /**
   * Platform names.
   * @typedef {Object} PlatformNames
   * @enum
   */

  var PlatformNames = Object.freeze({
    TRUE: 'truetv'
  });
  /**
   * Platform types.
   * @typedef {Object} PlatformTypes
   * @enum
   */

  var PlatformTypes = Object.freeze({
    CMS_ID: 'cmsid'
  });
  /**
   * Player events.
   * @typedef {Object} PlayerEvents
   * @enum
   */

  var PlayerEvents = Object.freeze({
    ENDED: 'ended',
    ERROR: 'error',
    PAUSE: 'pause',
    PLAY: 'play',
    PLAYING: 'playing',
    TIME_UPDATE: 'timeupdate'
  });
  /**
   * PTV SDK events.
   * @typedef {Object} SdkEvents
   * @enum
   */

  var SdkEvents = Object.freeze({
    CLICK_MISS: 'ptv.click.miss',
    CONFIG_FAILURE: 'ptv.config.failure',
    CONFIG_READY: 'ptv.config.ready'
  });

  /**
   * Source identifier for post messages.
   * @type {string}
   */

  var source = '@ptv-host';
  /**
   * Parse a message received from postMessage.
   *
   * @param {*} data The data received from postMessage.
   * @return {Object} Return parsed object
   */

  function parseMessageData(data) {
    if (typeof data === 'string') {
      try {
        data = JSON.parse(data);
      } catch (error) {
        return {};
      }
    }

    return data;
  }
  /**
   * Post a message to the specified target.
   *
   * @param {PtvEmbed} iframe Instance of the PtvEmbed class.
   * @param {string} method The API method to call.
   * @param {Object} params The parameters to send to the player.
   * @return {void}
   */

  function postMessage(iframe, method, params) {
    if (!iframe || !iframe.el || !iframe.el.contentWindow || !iframe.el.contentWindow.postMessage) {
      return;
    }

    var message = {
      method: method,
      source: source
    };

    if (params !== undefined) {
      message.value = params;
    } // IE 8 and 9 do not support passing messages, so stringify them.


    var ieVersion = parseFloat(window__default['default'].navigator.userAgent.toLowerCase().replace(/^.*msie (\d+).*$/, '$1'));

    if (ieVersion >= 8 && ieVersion < 10) {
      message = JSON.stringify(message);
    }

    iframe.el.contentWindow.postMessage(message, iframe.origin);
  }

  /**
   * Converts object to query string.
   *
   * @param {Object} obj Object to serialize
   * @return {string} Query string
   */
  var serialize = function serialize(obj) {
    return Object.keys(obj).filter(function (k) {
      return obj[k] !== null;
    }).map(function (k) {
      return encodeURIComponent(k) + "=" + encodeURIComponent(obj[k]);
    }).join('&');
  };

  var PROTOCOL = 'https://';
  var defaultCallbacks = {
    onClickMiss: Function.prototype,
    onConfigFailure: Function.prototype,
    onConfigReady: Function.prototype
  };
  var requiredOptions = {
    embedHost: null,
    excludePlayer: true,
    iframe: true
  };

  var createDefaultPreloadState = function createDefaultPreloadState() {
    return {
      config: undefined,
      started: undefined,
      visible: undefined,
      time: -1
    };
  };
  /**
   * Class that wraps the iframe element that loads the PTV canvas page and
   * exposes the SDK API to the videojs plugin.
   */


  var PtvEmbed = /*#__PURE__*/function () {
    /**
     * We make the actual assignment of the iframe source a static function to allow
     * tests to easily override the implementation (e.g., to set some custom
     * iframe content).
     *
     * @param {HTMLIframeElement} el The iframe DOM element
     * @param {string} origin The origin of the iframe content
     * @param {Object} config The SDK config to be passed as URL params to the SDK
     */
    PtvEmbed.assignIframeSource = function assignIframeSource(el, origin, config) {
      el.setAttribute('src', origin + '?' + serialize(config));
    }
    /**
     * This static function is overriden in tests when mock iframe content is set
     * via `PtvEmbed.assignIframeSource`.
     *
     * @param {Object} options Embed options.
     * @return {string} Returns full-qualified embed host URL.
     */
    ;

    PtvEmbed.getOrigin = function getOrigin(options) {
      return PROTOCOL + options.embedHost;
    }
    /**
     * Main constructor function.
     *
     * @param {Object} options Embed options to be serialized.
     * @param {Object} callbacks Map of callback events to execute on message
     * @param {string} iframeContent The optional content for the iframe (useful for tests)
     *  received.
     */
    ;

    function PtvEmbed(options, callbacks) {
      // Initialize private instance properties.
      this.isSdkLoaded_ = false;
      this.preloadState_ = createDefaultPreloadState(); // Generate config options.

      this.config_ = videojs__default['default'].mergeOptions(options, requiredOptions);
      this.origin_ = PtvEmbed.getOrigin(options); // Create iFrame element.

      this.el_ = window__default['default'].document.createElement('iframe');
      this.el_.onload = this.onLoad_.bind(this);
      this.el_.className = 'ptv-iframe';
      this.el_.style.cssText = "\n      position: absolute;\n      top: 0;\n      left: 0;\n      width: 100%;\n      height: 100%;\n      max-width: 100%;\n      max-height: 100%;\n      border: none;\n    "; // Assign iframe source to embed.

      PtvEmbed.assignIframeSource(this.el_, this.origin_, this.config_); // Store callbacks from plugin.

      this.callbacks_ = videojs__default['default'].mergeOptions(defaultCallbacks, callbacks); // Setup post message interface.

      this.handleMessage_ = this.handleMessage_.bind(this);
      window__default['default'].addEventListener('message', this.handleMessage_);
    }
    /**
     * Dispose the embed
     */


    var _proto = PtvEmbed.prototype;

    _proto.dispose = function dispose() {
      window__default['default'].removeEventListener('message', this.handleMessage_);
      this.callbacks_ = defaultCallbacks;
    }
    /**
     * Callbacks getter
     *
     * @return {Object} Map of event callbacks.
     */
    ;

    /**
     * Destroy instance and remove iframe element.
     */
    _proto.destroy = function destroy() {
      this.el.parentNode.removeChild(this.el);
      this.el_ = null;
      this.isSdkLoaded_ = undefined;
    }
    /**
     * Hide overlays.
     */
    ;

    _proto.hide = function hide() {
      if (this.isSdkLoaded_) {
        this.callMethod_('hide');
      } else {
        this.preloadState_.visible = false;
      }
    }
    /**
     * Stop any running PTV instance and load new config.
     *
     * @param {Object} config Config object passed to ptv.js
     */
    ;

    _proto.load = function load(config) {
      if (this.isSdkLoaded_) {
        this.callMethod_('load', config);
      } else {
        this.preloadState_.config = config;
      }
    }
    /**
     * Event emitter `off` method.
     *
     * @param {string} eventName Event name.
     */
    ;

    _proto.off = function off(eventName) {
      this.callMethod_('off', eventName);
    }
    /**
     * Event emitter `on` method.
     *
     * @param {string} eventName Event name.
     */
    ;

    _proto.on = function on(eventName) {
      this.callMethod_('on', eventName);
    }
    /**
     * Event emitter `once` method.
     *
     * @param {string} eventName Event name.
     */
    ;

    _proto.once = function once(eventName) {
      this.callMethod_('once', eventName);
    }
    /**
     * Event emitter `removeAllListeners` method.
     *
     * @param {Array} eventNames Array of event names.
     */
    ;

    _proto.removeAllListeners = function removeAllListeners(eventNames) {
      this.callMethod_('removeAllListeners', eventNames);
    }
    /**
     * Show overlays.
     */
    ;

    _proto.show = function show() {
      if (this.isSdkLoaded_) {
        this.callMethod_('show');
      } else {
        this.preloadState_.visible = true;
      }
    }
    /**
     * Start and show overlays.
     */
    ;

    _proto.start = function start() {
      if (this.isSdkLoaded_) {
        this.callMethod_('start');
      } else {
        this.preloadState_.started = true;
      }
    }
    /**
     * Stop and hide overlays.
     */
    ;

    _proto.stop = function stop() {
      if (this.isSdkLoaded_) {
        this.callMethod_('stop');
      } else {
        this.preloadState_.started = false;
      }
    }
    /**
     * Update play head.
     *
     * @param {number} seconds Player playhead in seconds.
     */
    ;

    _proto.timeUpdate = function timeUpdate(seconds) {
      if (this.isSdkLoaded_) {
        this.callMethod_('timeUpdate', seconds);
      } else {
        this.preloadState_.time = seconds;
      }
    }
    /**
     * Get a promise for a method.
     *
     * @param {string} name The API method to call.
     * @param {*} args Arguments to send via postMessage.
     * @todo We should consider making this promised based.
     * @private
     */
    ;

    _proto.callMethod_ = function callMethod_(name, args) {
      postMessage(this, name, args);
    }
    /**
     * Handles clicks that fall through to the root of the iFrame, indicating no
     * overlays were interacted with.
     *
     * @param {Object} message Message sent from iframe postMessage.
     * @private
     */
    ;

    _proto.handleMessage_ = function handleMessage_(message) {
      var data = message.data,
          origin = message.origin;

      if (origin === this.origin_) {
        var payload = parseMessageData(data); // If we receive any kind of message then we know the SDK has loaded.

        if (!this.isSdkLoaded_) {
          this.isSdkLoaded_ = true;
          this.applyPreloadState_();
        }

        switch (payload.type) {
          case SdkEvents.CONFIG_FAILURE:
            this.callbacks.onConfigFailure();
            break;

          case SdkEvents.CONFIG_READY:
            this.callbacks.onConfigReady(payload.data);
            break;

          case SdkEvents.CLICK_MISS:
            this.callbacks.onClickMiss();
            break;

          default:
            return;
        }
      }
    }
    /**
     * Handle iFrame onload event. Destroys instance if any errors occur.
     *
     * @todo This should be comprehensive to avoid any negative side-effects.
     * @private
     */
    ;

    _proto.onLoad_ = function onLoad_() {
      try {
        if (this.el.contentDocument.body.innerText === 'NOT FOUND') {
          this.destroy();
        }
      } catch (e) {// Could log this error, if needed.
      }
    }
    /**
     * Applies any state changes that occurred before the SDK was ready.
     *
     * @private
     */
    ;

    _proto.applyPreloadState_ = function applyPreloadState_() {
      var _this$preloadState_ = this.preloadState_,
          config = _this$preloadState_.config,
          started = _this$preloadState_.started,
          time = _this$preloadState_.time,
          visible = _this$preloadState_.visible;

      if (!this.isSdkLoaded_) {
        return;
      }

      if (config) {
        this.load(config);
      }

      if (started === true) {
        this.start();
      }

      if (started === false) {
        this.stop();
      }

      if (visible === true) {
        this.show();
      }

      if (visible === false) {
        this.hide();
      }

      if (time > -1) {
        this.timeUpdate(time);
      } // Revert properties to defaults.


      this.preloadState_ = createDefaultPreloadState();
    };

    createClass(PtvEmbed, [{
      key: "callbacks",
      get: function get() {
        return this.callbacks_;
      }
      /**
       * Element getter
       *
       * @return {HTMLIFrameElement} Map of event callbacks.
       */

    }, {
      key: "el",
      get: function get() {
        return this.el_;
      }
      /**
       * Origin getter
       *
       * @return {Object} Origin of embed.
       */

    }, {
      key: "origin",
      get: function get() {
        return this.origin_;
      }
    }]);

    return PtvEmbed;
  }();

  var version = "1.5.0";

  var Plugin = videojs__default['default'].getPlugin('plugin'); // Default options for the plugin.

  var defaults = {
    apiHost: ApiHosts.PRODUCTION,
    autoWireEvents: true,
    channelId: null,
    debug: false,
    enableGeoBlock: false,
    embedHost: EmbedHosts.PRODUCTION,
    platformId: null,
    platformName: null,
    platformType: null,
    previewMode: false,
    showDebugPanel: false,
    showPoster: false,
    streamId: null,
    viewerId: null,
    viewerLatitude: null,
    viewerLongitude: null
  };
  var defaultState = {
    configReady: false,
    configFailure: false
  };
  /**
   * An advanced Video.js plugin. For more information on the API
   *
   * See: https://blog.videojs.com/feature-spotlight-advanced-plugins/
   */

  var Ptv = /*#__PURE__*/function (_Plugin) {
    inheritsLoose(Ptv, _Plugin);

    /**
     * Create a Ptv plugin instance.
     *
     * @param  {Player} player
     *         A Video.js Player instance.
     *
     * @param  {Object} [options]
     *         An optional options object.
     *
     *         While not a core part of the Video.js plugin architecture, a
     *         second argument of options is a convenient way to accept inputs
     *         from your plugin's caller.
     */
    function Ptv(player, options) {
      var _this;

      // the parent class will add player under this.player
      _this = _Plugin.call(this, player) || this;

      _this.setState(defaultState);

      _this.options = videojs__default['default'].mergeOptions(defaults, options); // Handle player events.

      _this.player.ready(_this.handlePlayerReady_.bind(assertThisInitialized(_this)));

      return _this;
    }
    /**
     * Handles primary player ready event.
     */


    var _proto = Ptv.prototype;

    _proto.handlePlayerReady_ = function handlePlayerReady_() {
      var callbacks = {
        onClickMiss: this.handleClickMiss_.bind(this),
        onConfigReady: this.handleConfigReady_.bind(this),
        onConfigFailure: this.handleConfigFailure_.bind(this)
      };
      this.player.addClass('vjs-ptv'); // Create iframe instance.

      this.embed = new PtvEmbed(this.options, callbacks); // Setup other player events

      this.addPlayerListeners_(); // Place iFrame after video element and before the poster image element.

      this.player.posterImage.el().before(this.embed.el);
    }
    /**
     * Handles clicks that fall through to the root of the iFrame, indicating no
     * overlays were interacted with.
     */
    ;

    _proto.handleClickMiss_ = function handleClickMiss_() {
      if (this.player.bigPlayButton && this.player.bigPlayButton.enabled_) {
        this.togglePlay_();
      }
    }
    /**
     * Handles SDK config failure.
     */
    ;

    _proto.handleConfigFailure_ = function handleConfigFailure_() {
      this.setState({
        configFailure: true
      });

      if (this.embed) {
        this.embed.destroy();
      }
    }
    /**
     * Handles SDK config loaded.
     *
     * @param {Object}  data Response from the PTV config API.
     * @param {boolean} data.poster Poster set in the Promethean Ignite Video Platform.
     * @param {string}  data.src Video source set in the Promethean Ignite Video Platform.
     * @param {string}  data.type Video type set in the Promethean Ignite Video Platform.
     */
    ;

    _proto.handleConfigReady_ = function handleConfigReady_(_ref) {
      var poster = _ref.poster,
          src = _ref.src,
          type = _ref.type;
      this.setState({
        configReady: true
      }); // Start SDK if video already playing.

      if (!this.player.paused()) {
        this.start();
      } // Use poster from API.


      if (this.options.showPoster && typeof poster === 'object') {
        this.player.poster(poster.loading);
      } // Use video from API, if no video playing.


      if (!this.player.src() && this.player.canPlayType(type)) {
        this.player.src(src);
      }
    }
    /**
     * Method to toggle play / pause. Used to simulate BigPlayButton behavior when
     * overlays are not interacted with since the iFrame consumes all clicks.
     */
    ;

    _proto.togglePlay_ = function togglePlay_() {
      if (this.player.paused()) {
        this.player.play();
      } else {
        this.player.pause();
      }
    }
    /**
     * Add videojs player listeners.
     */
    ;

    _proto.addPlayerListeners_ = function addPlayerListeners_() {
      var _this2 = this;

      if (!this.options.autoWireEvents) {
        return;
      }

      this.player.one(PlayerEvents.PLAYING, function () {
        return _this2.start();
      });
      this.player.on(PlayerEvents.ENDED, function () {
        return _this2.stop();
      });
      this.player.on(PlayerEvents.ERROR, function () {
        return _this2.stop();
      });
      this.player.on(PlayerEvents.PAUSE, function () {
        return _this2.hide();
      });
      this.player.on(PlayerEvents.PLAY, function () {
        return _this2.show();
      });
      this.player.on(PlayerEvents.TIME_UPDATE, function () {
        return _this2.timeUpdate(_this2.player.currentTime());
      });
    }
    /**
     * Remove videojs player listeners.
     */
    ;

    _proto.removePlayerListeners_ = function removePlayerListeners_() {
      if (!this.options.autoWireEvents) {
        return;
      }

      this.player.off([PlayerEvents.ENDED, PlayerEvents.ERROR, PlayerEvents.PAUSE, PlayerEvents.PLAY, PlayerEvents.PLAYING, PlayerEvents.TIME_UPDATE]);
    }
    /**
     * Dispose the plugin
     */
    ;

    _proto.dispose = function dispose() {
      this.removePlayerListeners_();

      if (this.embed) {
        this.embed.dispose();
      }

      _Plugin.prototype.dispose.call(this);
    } // //////////////////////////
    // Public API
    //
    // NOTE: These need to remain for now to have parity with the SDK API.

    /**
     * Hide overlays.
     */
    ;

    _proto.hide = function hide() {
      if (this.embed) {
        this.embed.hide();
      }
    }
    /**
     * Stop any running PTV instance and load new config.
     *
     * @param {Object} config Config object passed to ptv.js
     */
    ;

    _proto.load = function load(config) {
      this.setState(defaultState);

      if (this.embed) {
        this.embed.load(config);
      }
    }
    /**
     * Show overlays.
     */
    ;

    _proto.show = function show() {
      if (this.embed) {
        this.embed.show();
      }
    }
    /**
     * Start and show overlays.
     */
    ;

    _proto.start = function start() {
      if (this.embed) {
        this.embed.start();
      }
    }
    /**
     * Stop and hide overlays.
     */
    ;

    _proto.stop = function stop() {
      if (this.embed) {
        this.embed.stop();
      }
    }
    /**
     * Update play head.
     *
     * @param {number} seconds Player playhead in seconds.
     */
    ;

    _proto.timeUpdate = function timeUpdate(seconds) {
      if (this.embed) {
        this.embed.timeUpdate(seconds);
      }
    };

    return Ptv;
  }(Plugin); // Define default values for the plugin's `state` object here.


  Ptv.defaultState = defaultState; // Include the version number.

  Ptv.VERSION = version; // Register the plugin with video.js.

  videojs__default['default'].registerPlugin('ptv', Ptv); // Create and export static types.

  var PtvTypes = window__default['default'].PtvTypes = {
    ApiHosts: ApiHosts,
    EmbedHosts: EmbedHosts,
    PlatformNames: PlatformNames,
    PlatformTypes: PlatformTypes
  };

  exports.PtvTypes = PtvTypes;
  exports.default = Ptv;

  Object.defineProperty(exports, '__esModule', { value: true });

})));
